/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.funclib.dambach.konfiguration.netz;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse zur Realisierung eines Straenknotens
 * @author Dambach Werke GmbH
 */
public class StrassenKnoten
{
  /**
   * Zum Strassenknoten gehrendes Systemobjekt
   */
  private SystemObject m_systemObject;
  
  /**
   * Liste der usseren Straensegmente die von diesem Knoten abgehen
   */
  private List<AeusseresStrassenSegment> m_abgehendeAeussereStrassensegmente = new ArrayList<AeusseresStrassenSegment>();
 
  /**
   * Liste der usseren Straensegmente die zu diesem Knoten hinfhren
   */
  private List<AeusseresStrassenSegment> m_hinfuehrendeAeussereStrassensegmente = new ArrayList<AeusseresStrassenSegment>();
  
  /**
   * Liste der inneren Straensegmente dieses Knotens
   */
  private List<InneresStrassenSegment> m_innereStrassenSegmente = new ArrayList<InneresStrassenSegment>();
  
  /**
   * Typ des Straenknotens
   */
  private String m_typ = null;
  
  /**
   * Konstruktor der Klasse
   * @param connection Verbindung zum DaV
   * @param systemObject Objekt vom Typ "typ.straenKnoten"
   */
  public StrassenKnoten (ClientDavInterface connection, SystemObject systemObject)
  {
    m_systemObject = systemObject;
    
    DataModel dataModel = connection.getDataModel();

    AttributeGroup atg = dataModel.getAttributeGroup("atg.straenKnoten");
        
    Data data = systemObject.getConfigurationData (atg);

    if (data != null)
      m_typ = data.getTextValue( "Typ" ).getValueText();
  }

  /**
   * Methode fgt ein abgehendes ueres Straensegment zum Knoten hinzu  
   * @param objekt ueres Straensegment
   */
  public void addAbgehendesAeusseresStrassenSegment( AeusseresStrassenSegment objekt )
  {
    if (!m_abgehendeAeussereStrassensegmente.contains( objekt ))
      m_abgehendeAeussereStrassensegmente.add( objekt );
  }

  /**
   * Methode fgt ein hinfhrendes ueres Straensegment zum Knoten hinzu  
   * @param objekt ueres Straensegment
   */
  public void addHinfuehrendesAeusseresStrassenSegment( AeusseresStrassenSegment objekt )
  {
    if (!m_hinfuehrendeAeussereStrassensegmente.contains( objekt ))
      m_hinfuehrendeAeussereStrassensegmente.add( objekt );
  }

  /**
   * Methode liefert eine Liste aller von diesem Knoten abgehender uerer Straensegmente
   * @return Liste mit ueren Straensegmenten
   */
  public List<AeusseresStrassenSegment> getAbgehendeAeussereStrassenSegmente ()
  {
    return m_abgehendeAeussereStrassensegmente;
  }
  
  /**
   * Methode liefert eine Liste aller an diesem Knoten hinfhrender uerer Straensegmente
   * @return Liste mit ueren Straensegmenten
   */
  public List<AeusseresStrassenSegment> getHinfuehrendeAeussereStrassenSegmente ()
  {
    return m_hinfuehrendeAeussereStrassensegmente;   
  }

  /**
   * Methode liefert alle inneren Staensegmente dieses Knotens
   * @return Liste mit inneren Straensegmenten
   */
  public List<InneresStrassenSegment> getInnereStrassenSegmente ()
  {
    return m_innereStrassenSegmente;
  }
  
  /**
   * @return liefert die Variable systemObject zurck
   */
  public SystemObject getSystemObject()
  {
    return m_systemObject;
  }

  /**
   * Methode fgt ein inneres Straensegment dem Knoten hinzu
   * @param objekt inneres Straensegment
   */
  public void addInneresStrassenSegment( InneresStrassenSegment objekt )
  {
    if (!m_innereStrassenSegmente.contains( objekt ))
      m_innereStrassenSegmente.add( objekt );
  }

  /**
   * @return liefert die Variable typ zurck
   */
  public String getTyp()
  {
    return m_typ;
  }

  /**
   * Methode prft ob es sich bei dem Knoten um ein Autobahnkreuz handelt
   * @return Knoten ist Autobahnkreuz: true, sonst false
   */
  public boolean isAutobahnKreuz ()
  {
    return m_typ.equals( "AutobahnKreuz" );
  }
  
  /**
   * Methode prft ob es sich bei dem Knoten um ein Autobahndreieck handelt
   * @return Knoten ist Autobahndreieck: true, sonst false
   */
  public boolean isAutobahnDreieck ()
  {
    return m_typ.equals( "AutobahnDreieck" );
  }

  /**
   * Methode prft ob es sich bei dem Knoten um ein Autobahnende handelt
   * @return Knoten ist Autobahnende: true, sonst false
   */
  public boolean isAutobahnEnde ()
  {
    return m_typ.equals( "AutobahnEnde" );
  }
  
  /**
   * Methode prft ob es sich bei dem Knoten um eine Autobahnanschlussstelle handelt
   * @return Knoten ist eine Autobahnanschlussstelle: true, sonst false
   */
  public boolean isAutobahnAnschlussStelle ()
  {
    return m_typ.equals( "AutobahnAnschlussStelle" );
  }
  
  /**
   * Methode prft ob es sich bei dem Knoten um einen sonstigen Knoten handelt
   * @return Knoten ist ein sonstiger Knoten: true, sonst false
   */
  public boolean isSonstigerKnoten ()
  {
    return m_typ.equals( "SonstigerKnoten" );
  }
  
  /**
   * Methode liefert die Pid des Systemobjekts, das zu diesem Straenknoten gehrt
   * @return Pid im Fehlerfall null
   */
  public String getPid ()
  {
    if (m_systemObject != null)
      return m_systemObject.getPid();
    
    return null;
  }
}
